from __future__ import annotations

import typing
from dataclasses import dataclass


@dataclass(init=False, repr=False)
class Assignment:
    parentCategory: Category
    objectSectionID: int
    parentObjectSectionID: None | typing.Any
    type: int
    personID: int
    taskID: int
    groupActivityID: int
    termIDs: list[int]
    assignmentName: str
    calendarID: int
    structureID: int
    sectionID: int
    dueDate: str
    assignedDate: str
    modifiedDate: str
    courseName: str
    active: bool
    scoringType: str
    score: str
    scorePoints: str
    scorePercentage: str
    totalPoints: int
    comments: None | typing.Any
    feedback: None | typing.Any
    late: bool
    missing: bool
    cheated: bool
    dropped: bool
    incomplete: bool
    turnedIn: bool
    wysiwygSubmission: bool
    upload: bool
    driveSubmission: bool
    multiplier: float
    hasStudentHTML: None | typing.Any
    hasTeacherHTML: None | typing.Any
    hasQuiz: None | typing.Any
    hasLTI: None | typing.Any
    hasLTIAcceptsScores: None | typing.Any
    hasFile: None | typing.Any
    hasExternalFile: None | typing.Any
    hasSubmission: None | typing.Any
    hasDiscussion: None | typing.Any
    hasRubric: None | typing.Any
    notGraded: bool
    isValidRubric: bool
    isValidMark: bool
    includeCampusLearning: bool

    def __init__(self, **kwargs) -> None:
        for kwarg in kwargs.keys():
            setattr(self, kwarg, kwargs[kwarg])

    def __getattr__(self, __name) -> None:
        return None

    def __repr__(self):
        return "{}: {}.".format(self.assignmentName, self.scorePercentage)


@dataclass(init=False, repr=False)
class Category:
    assignments: list[Assignment]
    categoryID: int
    sectionID: int
    name: str
    curveID: None | typing.Any
    weight: float
    seq: int
    hidePortal: None | typing.Any
    calcExclude: bool
    groupCourseID: None | typing.Any
    modifiedByID: int
    modifiedDate: str
    districtManaged: bool
    districtSourced: bool

    def __init__(self, **kwargs) -> None:
        for kwarg in kwargs.keys():
            setattr(self, kwarg, kwargs[kwarg])

    def __getattr__(self, __name) -> None:
        return None

    def __repr__(self):
        return f"{self.name} (weight: {self.weight})"



from __future__ import annotations

from .notifications import Notification
from .assignments import Assignment, Category
from .terms import Term, Course
from .district import District

from xml.etree import ElementTree


import requests
import json
import threading

SUCCESS_MSG = "success"
HEADERS = {
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36"
}


class StudentException(BaseException):
    pass


class Student:
    """Student class for Infinite Campus students"""

    def __init__(
        self, district: str | District, state: str, username: str, password: str
    ) -> None:
        self.district_name = district
        self.state = state
        self.username = username
        self.password = password
        self.__session = requests.Session()
        self.terms_cache = None
        self.assignments_cache = None

    def log_in(self) -> None:
        """Attempts to log in to IC as student."""

        # Verify district details
        self.district = District(self.district_name, self.state)
        self.district.validate()

        # Veryify login details.
        user_reponse = self.__session.get(
            "{}/verify.jsp?nonBrowser=true&username={}&password={}&appName={}".format(
                self.district.district_baseurl,
                self.username,
                self.password,
                self.district.district_app_name,
            ),
            headers=HEADERS,
        )
        if not SUCCESS_MSG in user_reponse.text:
            raise StudentException("Error verifying student's login credentials")

    def get_notifications(self) -> list[Notification]:
        """
        Get all recent notifications from student.
        Returns a list of notification objects.
        """
        notifcation_response = self.__session.get(
            "{}prism?x=notifications.Notification-retrieve".format(
                self.district.district_baseurl
            ),
            headers=HEADERS,
        )
        # IC returns an xml file for notifications that we convert to JSON.
        notifcation_response_json = json.loads(
            xmltojson.parse(notifcation_response.text)
        )
        notification_json_list = notifcation_response_json["campusRoot"][
            "NotificationList"
        ]["Notification"]
        notifications: list[Notification] = list()
        for notification_content in notification_json_list:
            notifications.append(
                Notification(
                    **{
                        # For most keys IC adds '@' as a prefix so we remove them.
                        attr.removeprefix("@"): value
                        for (attr, value) in notification_content.items()
                    }
                )
            )
        return notifications

    def get_assignments(self, store_cache: bool = True) -> list[Assignment]:
        """
        Get all assignments from the current school year from the student.
        'store_cache' indicates whether the assignments will be stored for
        faster access when calling other methods such as 'get_terms' which
        use the assignments.
        Returns a list of assignment objects.
        """
        assignment_response = self.__session.get(
            "{}api/portal/assignment/listView".format(self.district.district_baseurl),
            headers=HEADERS,
        )
        assignment_response_json = json.loads(assignment_response.text)
        assignments: list[Assignment] = list()
        for assignment_data in assignment_response_json:
            assignments.append(Assignment(**assignment_data))
        if store_cache and self.assignments_cache is None:
            self.assignments_cache = assignments
        return assignments

    def get_categories(
        self, course: Course, __store_categories: bool = False
    ) -> list[Category]:
        """
        Get all assignment categories for a course.
        Returns a list of category objects.
        """
        category_response = self.__session.get(
            "{}api/instruction/categories?sectionID={}".format(
                self.district.district_baseurl, course.sectionID
            ),
            headers=HEADERS,
        )
        category_response_json = json.loads(category_response.text)
        categories: list[Category] = list()
        if isinstance(category_response_json, dict):
            raise ValueError("section_id is invalid.")
        for category_data in category_response_json:
            categories.append(Category(**category_data))
        if __store_categories:
            course.categories = categories
        return categories

    def get_terms(self) -> list[Term]:
        """
        Get all terms from student.
        Returns a list of Term objects
        """
        term_response = self.__session.get(
            "{}resources/portal/grades/".format(self.district.district_baseurl),
            headers=HEADERS,
        )
        term_response_json = json.loads(term_response.text)
        term_json_list: list = term_response_json[0]["terms"]
        terms: list[Term] = list()
        assignments = self.assignments_cache
        if self.assignments_cache is None:
            assignments = self.get_assignments(store_cache=False)
        threads = list()
        for term in term_json_list:
            for i, course_data in enumerate(term["courses"]):
                # Convert course data to Course object to pass to 'get_categories'.
                course = Course(**course_data)
                term["courses"][i] = course
                t = threading.Thread(
                    target=self.get_categories,
                    args=(
                        course,
                        True,
                    ),
                )
                t.start()
                threads.append(t)
            [t.join() for t in threads]
            for course in term["courses"]:
                for category in course.categories:
                    category.assignments = list(
                        filter(
                            lambda assignment: assignment.sectionID == course.sectionID,
                            assignments,
                        )
                    )
                    for assignment in category.assignments:
                        assignment.parentCategory = category
            terms.append(Term(**term))
        return terms

    def get_gpa_percentage():
        ...



import requests
import json

from dataclasses import dataclass

ERR_MSG = "error"
HEADERS = {
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36"
}


class DistrictException(BaseException):
    pass


@dataclass(init=False)
class District:
    id: int
    district_name: str
    district_app_name: str
    district_baseurl: str
    district_code: str
    state_code: str
    staff_login_url: str
    parent_login_url: str
    student_login_url: str
    earliest: bool

    def __init__(self, distict_name, state_abrv):
        self.district_name = distict_name
        self.state_code = state_abrv
        self.__session = requests.session()

    def __getattr__(self, __name: str) -> None:
        return None

    def validate(self) -> None:
        """
        Raises an error if the district could not be found.
        Adds all attributes returned by IC.
        """
        district_response = self.__session.get(
            "https://mobile.infinitecampus.com/mobile/searchDistrict?query={}&state={}".format(
                self.district_name, self.state_code
            ),
            headers=HEADERS,
        )
        dist_response_json = json.loads(district_response.text)

        if ERR_MSG in dist_response_json:
            raise DistrictException(dist_response_json[ERR_MSG])

        if not "data" in dist_response_json.keys():
            raise KeyError("District data was not found")

        dist_data = dist_response_json["data"][0]
        for k, v in dist_data.items():
            setattr(self, k, v)



from __future__ import annotations

from .terms import Term


def calculate_gpa(terms: list[Term]) -> float:
    gpa = float()
    n_courses = 0
    for term in terms:
        if term.termName.lower() == "progress report":
            continue
        for course in term.courses:
            for grade in course.gradingTasks:
                if grade.percent:
                    gpa += grade.percent
                    n_courses += 1
    gpa /= n_courses
    return gpa


def number_grade(grade: float) -> str:
    return (grade / 20) - 1



from __future__ import annotations

from .assignments import Category

from dataclasses import dataclass


@dataclass(init=False, repr=False)
class Grade:
    _id: str
    personID: int
    trialID: int
    calendarID: int
    structureID: int
    courseID: int
    courseName: str
    sectionID: int
    taskID: int
    termID: int
    hasAssignments: bool
    hasCompositeTasks: bool
    taskName: str
    gradedOnce: bool
    treeTraversalSeq: int
    cumulativeTermSeq: int
    maxAssignments: int
    calcMethod: str
    groupWeighted: bool
    usePercent: bool
    curveID: int
    scoreID: int
    score: int
    percent: float
    comments: str
    progressScore: int
    progressPercent: float
    progressPointsEarned: float
    progressTotalPoints: float
    hasDetail: bool
    _model: str
    _hashCode: str
    termSeq: int
    termName: str

    def __init__(self, **kwargs) -> None:
        for kwarg in kwargs.keys():
            setattr(self, kwarg, kwargs[kwarg])

    def __repr__(self) -> str:
        return "{} {}".format(self.courseName, self.taskName)

    def __getattr__(self, _) -> None:
        # This is important one of these attributes does not exist, which is common
        return None


@dataclass(init=False, repr=False)
class Course:
    grades: list[Grade]
    categories: list[Category]
    _id: str
    rosterID: int
    personID: int
    structureID: int
    calendarID: int
    schoolID: int
    courseID: int
    sectionID: int
    courseName: str
    courseNumber: str
    isResponsive: bool
    sectionNumber: str
    endYear: int
    schoolName: str
    startDate: str
    trialID: int
    trialActive: bool
    roomName: str
    teacherDisplay: str
    ideStandardsOnPortal: bool
    dropped: bool

    def __init__(self, **kwargs) -> None:
        self.grades = list()
        for k, v in kwargs.items():
            if k != "gradingTasks":
                setattr(self, k, v)
        if "gradingTasks" in kwargs.keys():
            for grade_data in kwargs["gradingTasks"]:
                self.grades.append(Grade(**grade_data))

    def __repr__(self) -> str:
        return "{} {}".format(self.courseName, self.teacherDisplay)

    def __getattr__(self, _) -> None:
        return None


@dataclass(init=False, repr=False)
class Term:
    termID: int
    termName: str
    termScheduleID: int
    termScheduleName: str
    termSeq: int
    isPrimary: bool
    startDate: str
    endDate: str
    courses: list[Course]

    def __init__(self, **kwargs) -> None:
        self.courses = []
        for k, v in kwargs.items():
            if k != "courses":
                setattr(self, k, v)

        if "courses" in kwargs.keys():
            for course in kwargs["courses"]:
                if not isinstance(course, Course):
                    self.courses.append(Course(**course))
                    continue
                self.courses.append(course)

    def find_course(self, course_name: str) -> Course:
        for course in self.courses:
            if course.courseName == course_name:
                return course

        return None

    def __repr__(self) -> str:
        return "{}".format(self.termName)

    def __getattr__(self, _) -> None:
        return None




def dataclass_format(dataclass_name, _dict) -> str:
    attributes = []
    for k, v in _dict.items():
        attributes.append(f"{k}: {type(v).__name__}")
    return """
    @dataclass
    class %s:
        %s
    """ % (dataclass_name, "\n\t".join(attributes))



from dataclasses import dataclass


@dataclass(init=False, repr=False)
class DataObject:
    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self) -> str:
        return self.__class__.__name__


from __future__ import annotations

from dataclasses import dataclass


@dataclass(init=False, repr=False)
class Notification:
    """Notifcation class for student notifications."""

    notificationID: int
    userID: int
    creationTimestamp: str
    notificationTypeID: int
    read: bool
    notificationText: str
    notificationTypeText: str
    displayedDate: str
    finalText: str
    finalUrl: str

    def __init__(self, **kwargs) -> None:
        for kwarg in kwargs.keys():
            setattr(self, kwarg, kwargs[kwarg])

    def __repr__(self):
        return "{}: {}.".format(self.displayedDate, self.notificationText)
